@@ -17,9 +17,12 @@ use std::cmp::Ordering;
 use std::convert::{TryFrom, TryInto};
 use std::ffi::{CStr, CString};
 use std::fmt;
+use std::iter::{Product, Sum};
 use std::marker::PhantomData;
 use std::mem::MaybeUninit;
-use std::ops::Neg;
+use std::ops::{
+    Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign,
+};
 use std::str::FromStr;
 
 use libc::c_char;
@@ -32,7 +35,7 @@ use crate::decimal32::Decimal32;
 use crate::decimal64::Decimal64;
 use crate::error::{
     InvalidCoefficientError, InvalidExponentError, InvalidPrecisionError, ParseDecimalError,
-    TryFromDecimalError,
+    TryFromDecimalError, TryIntoDecimalError,
 };
 
 fn validate_n(n: usize) {
@@ -169,7 +172,7 @@ impl<const N: usize> Decimal<N> {
     /// The result is ordered with the least significant digits at index 0.
     ///
     /// [dpd]: http://speleotrove.com/decimal/dnnumb.html
-    pub(crate) fn coefficient_units(&self) -> &[u16] {
+    pub fn coefficient_units(&self) -> &[u16] {
         let units_len = Self::digits_to_lsu_elements_len(self.digits);
         &self.lsu[0..units_len]
     }
@@ -414,6 +417,148 @@ impl<const N: usize> PartialEq for Decimal<N> {
     }
 }
 
+impl<const N: usize> Neg for Decimal<N> {
+    type Output = Decimal<N>;
+
+    /// Note that this clones `self` to generate the negative value. For a
+    /// non-cloning method, use `Context::<N>::neg`.
+    fn neg(self) -> Decimal<N> {
+        let mut n = self.clone();
+        unsafe {
+            decnumber_sys::decNumberCopyNegate(n.as_mut_ptr(), n.as_ptr());
+        }
+        n
+    }
+}
+
+impl<const M: usize, const N: usize> Add<Decimal<M>> for Decimal<N> {
+    type Output = Self;
+
+    /// Note that this clones `self` to generate the output. For a
+    /// non-cloning method, use `Context::<N>::add`.
+    fn add(self, rhs: Decimal<M>) -> Self {
+        let mut lhs = self.clone();
+        Context::<Self>::default().add(&mut lhs, &rhs);
+        lhs
+    }
+}
+
+impl<const M: usize, const N: usize> AddAssign<Decimal<M>> for Decimal<N> {
+    fn add_assign(&mut self, rhs: Decimal<M>) {
+        Context::<Self>::default().add(self, &rhs);
+    }
+}
+
+impl<const M: usize, const N: usize> Div<Decimal<M>> for Decimal<N> {
+    type Output = Self;
+
+    /// Note that this clones `self` to generate the output. For a
+    /// non-cloning method, use `Context::<N>::div`.
+    fn div(self, rhs: Decimal<M>) -> Self {
+        let mut lhs = self.clone();
+        Context::<Self>::default().div(&mut lhs, &rhs);
+        lhs
+    }
+}
+
+impl<const M: usize, const N: usize> DivAssign<Decimal<M>> for Decimal<N> {
+    fn div_assign(&mut self, rhs: Decimal<M>) {
+        Context::<Self>::default().div(self, &rhs);
+    }
+}
+
+impl<const M: usize, const N: usize> Mul<Decimal<M>> for Decimal<N> {
+    type Output = Self;
+
+    /// Note that this clones `self` to generate the output. For a
+    /// non-cloning method, use `Context::<N>::mul`.
+    fn mul(self, rhs: Decimal<M>) -> Self {
+        let mut lhs = self.clone();
+        Context::<Self>::default().mul(&mut lhs, &rhs);
+        lhs
+    }
+}
+
+impl<const M: usize, const N: usize> MulAssign<Decimal<M>> for Decimal<N> {
+    fn mul_assign(&mut self, rhs: Decimal<M>) {
+        Context::<Self>::default().mul(self, &rhs);
+    }
+}
+
+impl<const M: usize, const N: usize> Rem<Decimal<M>> for Decimal<N> {
+    type Output = Self;
+
+    /// Note that this clones `self` to generate the output. For a
+    /// non-cloning method, use `Context::<N>::rem`.
+    fn rem(self, rhs: Decimal<M>) -> Self {
+        let mut lhs = self.clone();
+        Context::<Self>::default().rem(&mut lhs, &rhs);
+        lhs
+    }
+}
+
+impl<const M: usize, const N: usize> RemAssign<Decimal<M>> for Decimal<N> {
+    fn rem_assign(&mut self, rhs: Decimal<M>) {
+        Context::<Self>::default().rem(self, &rhs);
+    }
+}
+
+impl<const M: usize, const N: usize> Sub<Decimal<M>> for Decimal<N> {
+    type Output = Self;
+
+    /// Note that this clones `self` to generate the output. For a
+    /// non-cloning method, use `Context::<N>::sub`.
+    fn sub(self, rhs: Decimal<M>) -> Self {
+        let mut lhs = self.clone();
+        Context::<Self>::default().sub(&mut lhs, &rhs);
+        lhs
+    }
+}
+
+impl<const M: usize, const N: usize> SubAssign<Decimal<M>> for Decimal<N> {
+    fn sub_assign(&mut self, rhs: Decimal<M>) {
+        Context::<Self>::default().sub(self, &rhs);
+    }
+}
+
+impl<const M: usize, const N: usize> Sum<Decimal<M>> for Decimal<N> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = Decimal<M>>,
+    {
+        iter.map(|v| v).collect::<Vec<_>>().iter().sum()
+    }
+}
+
+impl<'a, const M: usize, const N: usize> Sum<&'a Decimal<M>> for Decimal<N> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = &'a Decimal<M>>,
+    {
+        let mut cx = Context::<Self>::default();
+        cx.sum(iter)
+    }
+}
+
+impl<const M: usize, const N: usize> Product<Decimal<M>> for Decimal<N> {
+    fn product<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = Decimal<M>>,
+    {
+        iter.map(|v| v).collect::<Vec<_>>().iter().product()
+    }
+}
+
+impl<'a, const M: usize, const N: usize> Product<&'a Decimal<M>> for Decimal<N> {
+    fn product<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = &'a Decimal<M>>,
+    {
+        let mut cx = Context::<Self>::default();
+        cx.product(iter)
+    }
+}
+
 impl<const N: usize> fmt::Debug for Decimal<N> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         fmt::Display::fmt(self, f)
@@ -449,16 +594,6 @@ impl<const N: usize> FromStr for Decimal<N> {
     }
 }
 
-impl<const N: usize> From<i32> for Decimal<N> {
-    fn from(n: i32) -> Decimal<N> {
-        let mut d = Decimal::default();
-        unsafe {
-            decnumber_sys::decNumberFromInt32(d.as_mut_ptr() as *mut decnumber_sys::decNumber, n);
-        }
-        d
-    }
-}
-
 /// Implements `std::convert::TryInto` semantics for `Decimal<N>` (represented
 /// by `$d`) into primitive integers (`$p`).
 macro_rules! __decnum_tryinto_primitive {
@@ -665,6 +800,16 @@ impl<const N: usize> From<f64> for Decimal<N> {
     }
 }
 
+impl<const N: usize> From<i32> for Decimal<N> {
+    fn from(n: i32) -> Decimal<N> {
+        let mut d = Decimal::default();
+        unsafe {
+            decnumber_sys::decNumberFromInt32(d.as_mut_ptr() as *mut decnumber_sys::decNumber, n);
+        }
+        d
+    }
+}
+
 impl<const N: usize> From<u32> for Decimal<N> {
     fn from(n: u32) -> Decimal<N> {
         let mut d = Decimal::default();
@@ -678,18 +823,50 @@ impl<const N: usize> From<u32> for Decimal<N> {
 impl<const N: usize> From<i64> for Decimal<N> {
     fn from(n: i64) -> Decimal<N> {
         let mut cx = Context::<Decimal<N>>::default();
-        let d = decimal_from_signed_int!(cx, n);
-        debug_assert!(!cx.status().any());
-        d
+        cx.from_i64(n)
     }
 }
 
 impl<const N: usize> From<u64> for Decimal<N> {
     fn from(n: u64) -> Decimal<N> {
         let mut cx = Context::<Decimal<N>>::default();
-        let d = decimal_from_unsigned_int!(cx, n);
-        debug_assert!(!cx.status().any());
-        d
+        cx.from_u64(n)
+    }
+}
+
+/// Generates a [`Decimal`] from an `i128` or fails if the result would be
+/// imprecise, e.g. has more than `N*3` digits of precision.
+///
+/// For an infallible version of this function, see
+/// [`Context<Decimal<N>>::from_i128`].
+impl<const N: usize> TryFrom<i128> for Decimal<N> {
+    type Error = TryIntoDecimalError;
+    fn try_from(n: i128) -> Result<Decimal<N>, Self::Error> {
+        let mut cx = Context::<Decimal<N>>::default();
+        let d = cx.from_i128(n);
+        return if cx.status().any() {
+            Err(TryIntoDecimalError)
+        } else {
+            Ok(d)
+        };
+    }
+}
+
+/// Generates a [`Decimal`] from a `u128` or fails if the result would be
+/// imprecise, e.g. has more than `N*3` digits of precision.
+///
+/// For an infallible version of this function, see
+/// [`Context<Decimal<N>>::from_u128`].
+impl<const N: usize> TryFrom<u128> for Decimal<N> {
+    type Error = TryIntoDecimalError;
+    fn try_from(n: u128) -> Result<Decimal<N>, Self::Error> {
+        let mut cx = Context::<Decimal<N>>::default();
+        let d = cx.from_u128(n);
+        return if cx.status().any() {
+            Err(TryIntoDecimalError)
+        } else {
+            Ok(d)
+        };
     }
 }
 
@@ -760,20 +937,6 @@ impl<const N: usize> From<Decimal128> for Decimal<N> {
     }
 }
 
-impl<const N: usize> Neg for Decimal<N> {
-    type Output = Decimal<N>;
-
-    /// Note that this clones `self` to generate the negative value. For a
-    /// non-allocating method, use `Context::<N>::neg`.
-    fn neg(self) -> Decimal<N> {
-        let mut n = self.clone();
-        unsafe {
-            decnumber_sys::decNumberCopyNegate(n.as_mut_ptr(), n.as_ptr());
-        }
-        n
-    }
-}
-
 impl<const N: usize> Default for Context<Decimal<N>> {
     fn default() -> Context<Decimal<N>> {
         let mut ctx = MaybeUninit::<decnumber_sys::decContext>::uninit();
@@ -911,7 +1074,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Adds `lhs` and `rhs`, storing the result in `lhs`.
-    pub fn add(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn add<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberAdd(
                 lhs.as_mut_ptr(),
@@ -924,7 +1087,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Carries out the digitwise logical and of `lhs` and `rhs`, storing
     /// the result in `lhs`.
-    pub fn and(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn and<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberAnd(
                 lhs.as_mut_ptr(),
@@ -936,7 +1099,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Divides `lhs` by `rhs`, storing the result in `lhs`.
-    pub fn div(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn div<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberDivide(
                 lhs.as_mut_ptr(),
@@ -948,7 +1111,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Divides `lhs` by `rhs`, storing the integer part of the result in `lhs`.
-    pub fn div_integer(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn div_integer<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberDivideInteger(
                 lhs.as_mut_ptr(),
@@ -971,7 +1134,12 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// The multiplication is carried out first and is exact, so this operation
     /// only has the one, final rounding.
-    pub fn fma(&mut self, x: &mut Decimal<N>, y: &Decimal<N>, z: &Decimal<N>) {
+    pub fn fma<const L: usize, const M: usize>(
+        &mut self,
+        x: &mut Decimal<N>,
+        y: &Decimal<L>,
+        z: &Decimal<M>,
+    ) {
         unsafe {
             decnumber_sys::decNumberFMA(
                 x.as_mut_ptr(),
@@ -983,6 +1151,26 @@ impl<const N: usize> Context<Decimal<N>> {
         }
     }
 
+    /// Constructs a number from an `i32`.
+    pub fn from_i32(&mut self, n: i32) -> Decimal<N> {
+        Decimal::<N>::from(n)
+    }
+
+    /// Constructs a number from an `i32`.
+    pub fn from_u32(&mut self, n: u32) -> Decimal<N> {
+        Decimal::<N>::from(n)
+    }
+
+    /// Constructs a number from an `i64`.
+    pub fn from_i64(&mut self, n: i64) -> Decimal<N> {
+        decimal_from_signed_int!(self, n)
+    }
+
+    /// Constructs a number from a `u64`.
+    pub fn from_u64(&mut self, n: u64) -> Decimal<N> {
+        decimal_from_unsigned_int!(self, n)
+    }
+
     /// Constructs a number from an `i128`.
     ///
     /// Note that this function can return inexact results for numbers with more
@@ -1221,7 +1409,7 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// The comparison is performed using the same rules as for
     /// [`total_cmp`](Context::<Decimal128>::total_cmp).
-    pub fn max(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn max<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberMax(
                 lhs.as_mut_ptr(),
@@ -1234,7 +1422,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Places whichever of `lhs` and `rhs` has the larger absolute value in
     /// `lhs`.
-    pub fn max_abs(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn max_abs<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberMaxMag(
                 lhs.as_mut_ptr(),
@@ -1249,7 +1437,7 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// The comparison is performed using the same rules as for
     /// [`total_cmp`](Context::<Decimal128>::total_cmp).
-    pub fn min(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn min<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberMin(
                 lhs.as_mut_ptr(),
@@ -1262,7 +1450,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Places whichever of `lhs` and `rhs` has the smaller absolute value in
     /// `lhs`.
-    pub fn min_abs(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn min_abs<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberMinMag(
                 lhs.as_mut_ptr(),
@@ -1281,7 +1469,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Multiples `lhs` by `rhs`, storing the result in `lhs`.
-    pub fn mul(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn mul<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberMultiply(
                 lhs.as_mut_ptr(),
@@ -1325,7 +1513,7 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// This operation is a generalization of the IEEE 754 *nextAfter*
     /// operation.
-    pub fn next_toward(&mut self, x: &mut Decimal<N>, y: &Decimal<N>) {
+    pub fn next_toward<const M: usize>(&mut self, x: &mut Decimal<N>, y: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberNextToward(
                 x.as_mut_ptr(),
@@ -1338,7 +1526,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Carries out the digitwise logical or of `lhs` and `rhs`, storing
     /// the result in `lhs`.
-    pub fn or(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn or<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberOr(
                 lhs.as_mut_ptr(),
@@ -1354,7 +1542,11 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// If either `lhs` or `rhs` is a NaN, returns `None`. To force an ordering
     /// upon NaNs, use [`total_cmp`](Context::<Decimal<N>>::total_cmp).
-    pub fn partial_cmp(&mut self, lhs: &Decimal<N>, rhs: &Decimal<N>) -> Option<Ordering> {
+    pub fn partial_cmp<const L: usize, const M: usize>(
+        &mut self,
+        lhs: &Decimal<L>,
+        rhs: &Decimal<M>,
+    ) -> Option<Ordering> {
         validate_n(N);
         let mut d = MaybeUninit::<Decimal<N>>::uninit();
         let d = unsafe {
@@ -1386,16 +1578,16 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Raises `x` to the power of `y`, storing the result in `x`.
-    pub fn pow(&mut self, x: &mut Decimal<N>, y: &Decimal<N>) {
+    pub fn pow<const M: usize>(&mut self, x: &mut Decimal<N>, y: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberPower(x.as_mut_ptr(), x.as_ptr(), y.as_ptr(), &mut self.inner);
         }
     }
 
     /// Takes product of elements in `iter`.
-    pub fn product<'a, I>(&mut self, iter: I) -> Decimal<N>
+    pub fn product<'a, I, const M: usize>(&mut self, iter: I) -> Decimal<N>
     where
-        I: Iterator<Item = &'a Decimal<N>>,
+        I: Iterator<Item = &'a Decimal<M>>,
     {
         iter.fold(Decimal::<N>::from(1), |mut product, d| {
             self.mul(&mut product, &d);
@@ -1405,7 +1597,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Rounds or pads `lhs` so that it has the same exponent as `rhs`, storing
     /// the result in `lhs`.
-    pub fn quantize(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn quantize<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberQuantize(
                 lhs.as_mut_ptr(),
@@ -1425,7 +1617,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Integer-divides `lhs` by `rhs`, storing the remainder in `lhs`.
-    pub fn rem(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn rem<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberRemainder(
                 lhs.as_mut_ptr(),
@@ -1438,7 +1630,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Like [`rem`](Context::<Decimal<N>>::rem), but uses the IEEE 754
     /// rules for remainder operations.
-    pub fn rem_near(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn rem_near<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberRemainderNear(
                 lhs.as_mut_ptr(),
@@ -1450,7 +1642,7 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Rescales `n` to have an exponent of `exp`.
-    pub fn rescale(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn rescale<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberRescale(
                 lhs.as_mut_ptr(),
@@ -1476,7 +1668,7 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// `rhs` specifies the number of positions to shift, and must be a finite
     /// integer.
-    pub fn shift(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn shift<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberShift(
                 lhs.as_mut_ptr(),
@@ -1494,7 +1686,7 @@ impl<const N: usize> Context<Decimal<N>> {
     ///
     /// `rhs` specifies the number of positions to rotate, and must be a finite
     /// integer.
-    pub fn rotate(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn rotate<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberRotate(
                 lhs.as_mut_ptr(),
@@ -1506,21 +1698,21 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Multiplies `x` by 10<sup>`y`</sup>, storing the result in `x`.
-    pub fn scaleb(&mut self, x: &mut Decimal<N>, y: &Decimal<N>) {
+    pub fn scaleb<const M: usize>(&mut self, x: &mut Decimal<N>, y: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberScaleB(x.as_mut_ptr(), x.as_ptr(), y.as_ptr(), &mut self.inner);
         }
     }
 
     /// Computes the square root of `n`, storing the result in `n`.
-    pub fn sqrt(&mut self, n: &mut Decimal<N>) {
+    pub fn sqrt<const M: usize>(&mut self, n: &mut Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberSquareRoot(n.as_mut_ptr(), n.as_ptr(), &mut self.inner);
         }
     }
 
     /// Subtracts `rhs` from `lhs`, storing the result in `lhs`.
-    pub fn sub(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn sub<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberSubtract(
                 lhs.as_mut_ptr(),
@@ -1532,9 +1724,9 @@ impl<const N: usize> Context<Decimal<N>> {
     }
 
     /// Sums all elements of `iter`.
-    pub fn sum<'a, I>(&mut self, iter: I) -> Decimal<N>
+    pub fn sum<'a, I, const M: usize>(&mut self, iter: I) -> Decimal<N>
     where
-        I: Iterator<Item = &'a Decimal<N>>,
+        I: Iterator<Item = &'a Decimal<M>>,
     {
         iter.fold(Decimal::<N>::zero(), |mut sum, d| {
             self.add(&mut sum, d);
@@ -1546,7 +1738,11 @@ impl<const N: usize> Context<Decimal<N>> {
     /// total order predicate defined in IEEE 754-2008.
     ///
     /// For a brief description of the ordering, consult [`f32::total_cmp`].
-    pub fn total_cmp(&mut self, lhs: &Decimal<N>, rhs: &Decimal<N>) -> Ordering {
+    pub fn total_cmp<const L: usize, const M: usize>(
+        &mut self,
+        lhs: &Decimal<L>,
+        rhs: &Decimal<M>,
+    ) -> Ordering {
         validate_n(N);
         let mut d = MaybeUninit::<Decimal<N>>::uninit();
         let d = unsafe {
@@ -1570,7 +1766,7 @@ impl<const N: usize> Context<Decimal<N>> {
 
     /// Carries out the digitwise logical xor of `lhs` and `rhs`, storing
     /// the result in `lhs`.
-    pub fn xor(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<N>) {
+    pub fn xor<const M: usize>(&mut self, lhs: &mut Decimal<N>, rhs: &Decimal<M>) {
         unsafe {
             decnumber_sys::decNumberXor(
                 lhs.as_mut_ptr(),
